/**************************************************************************//**
 * @file     ADC_Function.c
 * @version  V1.00
 * $Revision: 1 $
 * $Date: 14/01/22 11:27a $
 * @brief    NUC123 Series ADC Function Sample Code
 *
 * @note
 * Copyright (C) 2014 Nuvoton Technology Corp. All rights reserved.
 *
 ******************************************************************************/

/*!<Includes */
#include <stdio.h>
#include "NUC123Series.h"
#include "NucInternal.h"

/*--------------------------------------------------------------------------*/
/* Global variables */
__IO BOOL bIsADCInt = FALSE;
_ADC_INIT_T ADC_init;


/**
 * @brief       ADC interrupt handler
 * @param       None
 * @return      None
 * @details     ADC interrupt handler
 */
void ADC_IRQHandler(void)
{
    ADC_CLR_IF(ADC_CONV_DONE_IF);               //En:Clear ADC interrupt status

    bIsADCInt = TRUE;
}


/**
  * @brief  Initial ADC.
  * @param  None.
  * @retval None.
  */
void ADC_Init(void)
{
    ADC_ReInit(0x00);
}


/**
  * @brief  Initial ADC Function.
  * @param  setting:

  * @retval None.
  */
void ADC_ReInit(uint8_t setting)
{
    int i;

    SYS_RESET_IP2(SYS_RST_ADC);

    //Set GPIOD.0~3,6,7 to ADC  mode
    //ADC0~3
    outpw(&SYS->GPD_MFP, inpw(&SYS->GPD_MFP) | (0xF<<0));
    outpw(&SYS->ALT_MFP1, inpw(&SYS->ALT_MFP1) | (0xF<<16));

    //ADC6~7
    outpw(&SYS->GPF_MFP, inpw(&SYS->GPF_MFP) | (0x3<<2));
    outpw(&SYS->ALT_MFP1, inpw(&SYS->ALT_MFP1) | (0xF<<24));

    for(i=0; i<4; i++) {
        GPIO_SET_MODE(PD, i, GPIO_INPUT);
    }
    for(i=2; i<4; i++) {
        GPIO_SET_MODE(PF, i, GPIO_INPUT);
    }

    for(i=0; i<4; i++) {
        GPIO_DISABLE_DIGITAL_PATH(PD, i);
    }
    for(i=2; i<4; i++) {
        GPIO_DISABLE_DIGITAL_PATH(PF, i);
    }

    g_u8ParamSet[MODE_ADC] = setting;

}


/**
  * @brief  Disable ADC function
  * @param  None.
  * @retval None.
  */
void ADC_DeInit(void)
{
    int i;

    DrvADC_DisableInt();

    //Set GPIOD.0~3,6,7 to GPIO mode
    //ADC0~3
    outpw(&SYS->GPD_MFP, inpw(&SYS->GPD_MFP) & ~(0xF<<0));
    outpw(&SYS->ALT_MFP1, inpw(&SYS->ALT_MFP1) & ~(0xF<<16));

    //ADC6~7
    outpw(&SYS->GPF_MFP, inpw(&SYS->GPF_MFP) & ~(0x3<<2));
    outpw(&SYS->ALT_MFP1, inpw(&SYS->ALT_MFP1) & ~(0xF<<24));

    for(i=0; i<4; i++) {
        GPIO_ENABLE_DIGITAL_PATH(PD, i);
    }
    for(i=2; i<4; i++) {
        GPIO_ENABLE_DIGITAL_PATH(PF, i);
    }

    DrvADC_DeInit();
    g_u8ParamSet[MODE_ADC] = 0x00;
}


/**
  * @brief  ADC Main Function
  * @param  None.
  * @retval None.
  * @details Each ADC output would be the result of averaged value after sampled with BurstSampleCnt times.
  *          And would output to AP with TotalSampleCnt times.
  */

static __IO _ADC_FILTER_T ADC_Filter[8];
void ADC_MainProcess(void)
{
    static uint8_t u8BurstSampleCtrl = 0,u8TotalSampleCtrl = 0;
    int i;



    if(g_u16TxByte && g_u8ADCReady) {
        ADC_ReInit(0x12);

        SET_ADC_CLKSRC(g_u8TxBuf[g_u16TxHead++]);
        SET_ADC_CLKDIV(g_u8TxBuf[g_u16TxHead++]);

        APBCLK_ENABLE(CLK_ADC);
        ADC_CONVERTER_ENABLE();

        ADC_SET_MODE(ADC_SINGLE_SCAN);

        ADC_init.BurstSampleCnt = g_u8TxBuf[g_u16TxHead++];
        ADC_init.TotalSampleCnt = g_u8TxBuf[g_u16TxHead++];
        ADC_init.EnabledChannels = g_u8TxBuf[g_u16TxHead++];
        ADC_CH_ENABLE(ADC_init.EnabledChannels);

        u8BurstSampleCtrl = 0;
        u8TotalSampleCtrl = 0;

        memset((void *)ADC_Filter,0x00,sizeof(_ADC_FILTER_T)*8);

        DrvADC_EnableInt();   //En:Enable ADC interrupt

        ADC_START_CONVERSION();

        g_u16TxByte = 0;
        g_u16TxHead = 0;

        g_u8ADCReady = 0;
    }

    if(bIsADCInt == TRUE) {
//      uint8_t index = 0;
        bIsADCInt = FALSE;
        for(i=0; i<8; i++) {
            if((ADC_init.EnabledChannels&(1<<i)) == 0)
                continue;

            if(ADC_IS_RSLT_VALID(i)) {
                ADC_Filter[i].SampleCnt ++;
                ADC_Filter[i].DataBuffer += ADC_GET_RSLT(i);
            }
        }
        u8BurstSampleCtrl++;
        if(u8BurstSampleCtrl<ADC_init.BurstSampleCnt) {
            ADC_START_CONVERSION();
        } else {
            for(i=0; i<8; i++) {
                if(ADC_Filter[i].SampleCnt) {
                    ADC_Filter[i].DataBuffer =  ADC_Filter[i].DataBuffer/ADC_Filter[i].SampleCnt;
                    g_u8RxBuf[g_u16RxTail++] = (ADC_Filter[i].DataBuffer&0xFF);
                    g_u16RxTail &= RX_BOUND;
                    g_u16RxByte ++;

                    g_u8RxBuf[g_u16RxTail++] = (ADC_Filter[i].DataBuffer&0x3FF)>>8 | (i<<4);
                    g_u16RxTail &= RX_BOUND;
                    g_u16RxByte ++;
                }
            }

            //Send to Host
            u8TotalSampleCtrl++;
            if(u8TotalSampleCtrl<ADC_init.TotalSampleCnt) {
                u8BurstSampleCtrl = 0;
                memset((void *)ADC_Filter,0x00,sizeof(_ADC_FILTER_T)*8);
                ADC_START_CONVERSION();
            } else {
                DrvADC_DisableInt();
            }
        }
    }
}







