/**************************************************************************//**
 * @file     GPIO_Monitor.c
 * @version  V1.00
 * $Revision: 1 $
 * $Date: 13/11/11 11:27a $
 * @brief    NUC123 Series General Purpose I/O Monitor I2C Sample Code
 *
 * @note
 * Copyright (C) 2013 Nuvoton Technology Corp. All rights reserved.
 *
 ******************************************************************************/
#include <stdio.h>
#include "NucInternal.h"

#if(SOFTWARE_I2C_MONITOR == GPIO_MONITOR1)

/**
  * @brief  EINT1 main interrupt function.
  *             store received data in g_u8RxBuf, update related queue pointer and queue size
  *             received pattern 0x00: I2C data bit 0
  *             received pattern 0x01: I2C data bit 1
  * @param  None.
  * @retval None.
  */
void EINT1_IRQHandler(void)
{
    GPIO_CLR_PIN_IF(PB, 15);               //En:Clear PB.15 interrupt

    g_u8RxBuf[g_u16RxTail] = (PF->PIN & 0x0004) >> 2;

    g_u16RxTail++;
    g_u16RxTail &= RX_BOUND;
    g_u16RxByte++;

    if(g_u16RxByte >  RX_BOUND) {
        g_u16RxBufferOverflow = 1;
        NVIC_DisableIRQ(EINT1_IRQn);
        NVIC_DisableIRQ(GPCDF_IRQn);
    }
}

/**
  * @brief  GPCDF main interrupt function.
  *             store received data in g_u8RxBuf, update related queue pointer and queue size
  *             received pattern 0x02: I2C start bit
  *             received pattern 0x06: I2C stop bit
  * @param  None.
  * @retval None.
  */
void GPCDF_IRQHandler(void)
{
    GPIO_CLR_PIN_IF(PF, 2);

    // Check I2C Clock Pin (PB.15)
    if((PB->PIN & 0x8000)) {
        g_u8RxBuf[g_u16RxTail] = (PF->PIN & 0x0004) + 2;
        g_u16RxTail++;
        g_u16RxTail &= RX_BOUND;
        g_u16RxByte++;

        if(g_u16RxByte >  RX_BOUND) {
            g_u16RxBufferOverflow = 1;
            NVIC_DisableIRQ(EINT1_IRQn);
            NVIC_DisableIRQ(GPCDF_IRQn);
        }
    }
}

/**
  * @brief  Disable GPIO interrupt function
  * @param  None.
  * @retval None.
  */
void GPIO_DeInit(void)
{
    NVIC_DisableIRQ(EINT1_IRQn);
    NVIC_DisableIRQ(GPCDF_IRQn);
}

/**
  * @brief  Initial PB.6 to be software I2C data pin
  *             Initial PB.7 to be software I2C clock pin
  *             Enable GPIO interrupt to monitor PB.4 and PB.5
  * @param  None.
  * @retval None.
  */
void GPIO_Monitor_Init(void)
{
    GPIO_SET_MODE(PB, 15, GPIO_INPUT);                   //En:Configure PB.15 as input mode
    GPIO_EnableInt(PB, 15, GPIO_INT_RISING);             //set rising edge type of interrupt

    GPIO_SET_MODE(PF, 2, GPIO_INPUT);                   //En:Configure PB.6 as input mode
    GPIO_EnableInt(PF, 2, GPIO_INT_BOTH_EDGE);          //set both falling and rising edge type of interrupt

    NVIC_EnableIRQ(EINT1_IRQn);                          //En:Enable NVIC EINT1_IRQn interrupt
    NVIC_EnableIRQ(GPCDF_IRQn);                          //En:Enable NVIC GPCDF_IRQn interrupt
}

#endif  // #if(SOFTWARE_I2C_MONITOR == GPIO_MONITOR1)
