/**************************************************************************//**
 * @file     SPI2_Master.c
 * @version  V1.00
 * $Revision: 1 $
 * $Date: 13/11/11 11:27a $
 * @brief    NUC123 Series SPI2 Master Sample Code
 *
 * @note
 * Copyright (C) 2013 Nuvoton Technology Corp. All rights reserved.
 *
 ******************************************************************************/

/*!<Includes */
#include "NucInternal.h"


/**
  * @brief  Initial SPI2 to be SPI master with default setting 0xF4.
  * @param  None.
  * @retval None.
  */
void SPI2_Master_Init(void)
{
    // 12 MHz, SS Active Low, Order MSB first, Type 2
    SPI2_Master_ReInit(0xF4);
}

/**
  * @brief  Initial SPI2 to be SPI master.
  *              Set PB7~PB4 as SPI2 pins. PB4=SPISS20,PB5=SPICLK2,PB6=MOSI20,PB7=MISO20
  * @param  setting: the setting of SPI
  *                 bit 0~1 (SPI_TRANS_TYPE)
  *                     -00: Type0
  *                     -01: Type1
  *                     -10: Type2
  *                     -11: Type3
  *                 bit 2
  *                     -0: Least Significant Bit first
  *                     -1: Most Significant Bit first
  *                 bit 3
  *                     -0: SS active Low
  *                     -1: SS active High
  *                 bit  4~7 (clock rate)
  *                     -0000: 1 MHz
  *                     -0001: 2 MHz
  *                     -0010: 3 MHz
  *                     -0011: 4 MHz
  *                     -0100: 4.5 MHz
  *                     -0101: 6 MHz
  *                     -0111: 7.2 MHz
  *                     -1001: 9 MHz
  *                     -1111: 12 MHz
  *                     -others: Reserved
  * @retval None.
  */

void SPI2_Master_ReInit(uint8_t setting)
{

    GPIO_SPI2_PB();

    // Bit 0, Bit 1 for SPI_TRANS_TYPE
    switch(setting & 0x03) {
    case 0:
        DrvSPI_Init(SPI2, SPI_MASTER, SPI_TYPE0 , 8);
        break;

    case 1:
        DrvSPI_Init(SPI2, SPI_MASTER, SPI_TYPE1 , 8);
        break;

    case 2:
        DrvSPI_Init(SPI2, SPI_MASTER, SPI_TYPE2 , 8);
        break;

    case 3:
        DrvSPI_Init(SPI2, SPI_MASTER, SPI_TYPE3 , 8);
        break;
    }

    SPI_SET_DIVIDER(SPI2, 2);
    SET_SPI2_CLKSRC(CLK_SPI2_HCLK);

    switch((setting & 0xF0) >> 4) {
    case 0:
        DrvSPI_SetClockFreq(SPI2 , 1000000 , 0);
        break;

    case 1:
        DrvSPI_SetClockFreq(SPI2 , 2000000 , 0);
        break;

    case 2:
        DrvSPI_SetClockFreq(SPI2 , 3000000 , 0);
        break;

    case 3:
        DrvSPI_SetClockFreq(SPI2 , 4000000 , 0);
        break;

    case 4:
        DrvSPI_SetClockFreq(SPI2 , 4500000 , 0);
        break;

    case 5:
        DrvSPI_SetClockFreq(SPI2 , 6000000 , 0);
        break;

    case 7:
        DrvSPI_SetClockFreq(SPI2 , 7200000 , 0);
        break;

    case 9:
        DrvSPI_SetClockFreq(SPI2 , 9000000 , 0);
        break;

    case 15:
        DrvSPI_SetClockFreq(SPI2 , 12000000 , 0);
        break;
    }

    if(setting & 0x04)
        SPI_SET_MSB_FIRST(SPI2);
    else
        SPI_SET_LSB_FIRST(SPI2);


    if(setting & 0x08)
        SPI_SET_SS(SPI2, SPI_SS_HIGH_TRIGGER, SPI_SW_SS/*SPI_AUTO_SS*/); //En: Set slave auto select, high active
    else
        SPI_SET_SS(SPI2, SPI_SS_LOW_TRIGGER, SPI_SW_SS/*SPI_AUTO_SS*/); //En: Set slave auto select, high active

    g_u32SPI2ApbClock = DrvSPI_GetClock1Freq(SPI2);
    g_u8ParamSet[MODE_SPI2_MASTER] = setting;
}

/**
  * @brief  Disable SPI2 function and clock source. Set PB.4, PB.5, PB.6, and PB.7 as GPIO function
  * @param  None.
  * @retval None.
  */
void SPI2_Master_DeInit(void)
{
    DrvSPI_DeInit(SPI2);

    outpw(&SYS->GPB_MFP, inpw(&SYS->GPB_MFP) & ~(0x0F<<4));
    outpw(&SYS->ALT_MFP, inpw(&SYS->ALT_MFP) & ~(0x0F<<15));

    // Clear SPI2 Master recorded setting
    g_u8ParamSet[MODE_SPI2_MASTER] = 0x00;

}

/**
  * @brief  SPI2 main process function.
  *               transfer g_u8SPI2TxBuf  data to SPI slave device, update related queue pointer and queue size
  *              store received data in g_u8SPI2RxBuf, update related queue pointer and queue size
  * @param  None.
  * @retval None.
  */
void SPI2_Master_MainProcess(void)
{
    uint8_t     cmdCnt = 0;
    uint8_t     repeat = 0;
    uint8_t     bitLength = 0;

    uint8_t     dataCnt = 0;
    uint8_t     dataRxCnt = 0;

    uint16_t    idleTimeL = 0;
    uint16_t    idleTimeH = 0;
    uint16_t    DataIndex = 0;
    uint16_t    DataByte = 0;

    uint32_t    Data = 0;

    cmdCnt = g_u8SPI2TxBuf[g_u16SPI2TxHead++];
    g_u16SPI2TxByte--;

    while(cmdCnt) {
        // Check Valid Byte
        if('S' == g_u8SPI2TxBuf[g_u16SPI2TxHead++]) {
            repeat = g_u8SPI2TxBuf[g_u16SPI2TxHead++];
            idleTimeL = g_u8SPI2TxBuf[g_u16SPI2TxHead++];
            idleTimeH = g_u8SPI2TxBuf[g_u16SPI2TxHead++];
            idleTimeL += (idleTimeH << 8);

            g_u16SPI2TxByte -= 4;
        } else {
            // Error
            g_u16SPI2TxByte = 0;
            return;
        }

        DataIndex = g_u16SPI2TxHead;
        DataByte =  g_u16SPI2TxByte;


        while(repeat) {
            g_u16SPI2TxHead = DataIndex;
            g_u16SPI2TxByte = DataByte;

            // Start Transation
            SPI_SET_SS0(SPI2);

SPI_CHANGE_BIT_LENGTH:

            bitLength = g_u8SPI2TxBuf[g_u16SPI2TxHead++];
            dataCnt = g_u8SPI2TxBuf[g_u16SPI2TxHead++];
            dataRxCnt = dataCnt;

            SPI_DISABLE_FIFO(SPI2);
            SPI_SET_TX_BIT_LEN(SPI2, bitLength);
            SPI_CLEAR_TX_FIFO(SPI2);
            SPI_CLEAR_RX_FIFO(SPI2);
            SPI_SetFIFOMode(SPI2, 2);

            switch((bitLength + 7) >> 3) {
            case 1:
                while(dataCnt || dataRxCnt) {

                    while(dataCnt && (0 == SPI_GET_TX_FULL(SPI2))) {
                        Data = g_u8SPI2TxBuf[g_u16SPI2TxHead++];
                        g_u16SPI2TxByte--;
                        SPI_WRITE_TX0(SPI2, Data);
                        dataCnt--;

                        if(0 == SPI_GET_RX_EMPTY(SPI2))
                            break;
                    }

                    while(dataRxCnt) {
                        while(SPI_GET_RX_EMPTY(SPI2));
                        Data = SPI_GET_RX0_DATA(SPI2);
                        g_u8SPI2RxBuf[g_u16SPI2RxTail++] = Data & 0xFF;
                        g_u16SPI2RxTail &= RX_BOUND_PART;
                        g_u16SPI2RxByte++;
                        dataRxCnt--;
                    }

                }
                break;
            case 2:
                while(dataCnt || dataRxCnt) {

                    while(dataCnt && (0 == SPI_GET_TX_FULL(SPI2))) {

                        Data = g_u8SPI2TxBuf[g_u16SPI2TxHead++];
                        Data += (g_u8SPI2TxBuf[g_u16SPI2TxHead++] << 8);
                        g_u16SPI2TxByte -= 2;

                        SPI_WRITE_TX0(SPI2, Data);
                        dataCnt--;

                        if(0 == SPI_GET_RX_EMPTY(SPI2))
                            break;
                    }

                    while(dataRxCnt) {
                        while(SPI_GET_RX_EMPTY(SPI2));
                        Data = SPI_GET_RX0_DATA(SPI2);

                        g_u8SPI2RxBuf[g_u16SPI2RxTail++] = Data & 0xFF;
                        g_u16SPI2RxTail &= RX_BOUND_PART;

                        g_u8SPI2RxBuf[g_u16SPI2RxTail++] = (Data >> 8) & 0xFF;
                        g_u16SPI2RxTail &= RX_BOUND_PART;

                        g_u16SPI2RxByte += 2;
                        dataRxCnt--;
                    }

                }
                break;
            case 3:
                while(dataCnt || dataRxCnt) {

                    while(dataCnt && (0 == SPI_GET_TX_FULL(SPI2))) {

                        Data = g_u8SPI2TxBuf[g_u16SPI2TxHead++];
                        Data += (g_u8SPI2TxBuf[g_u16SPI2TxHead++] << 8);
                        Data += (g_u8SPI2TxBuf[g_u16SPI2TxHead++] << 16);
                        g_u16SPI2TxByte -= 3;

                        SPI_WRITE_TX0(SPI2, Data);
                        dataCnt--;

                        if(0 == SPI_GET_RX_EMPTY(SPI2))
                            break;
                    }

                    while(dataRxCnt) {
                        while(SPI_GET_RX_EMPTY(SPI2));
                        Data = SPI_GET_RX0_DATA(SPI2);

                        g_u8SPI2RxBuf[g_u16SPI2RxTail++] = Data & 0xFF;
                        g_u16SPI2RxTail &= RX_BOUND_PART;

                        g_u8SPI2RxBuf[g_u16SPI2RxTail++] = (Data >> 8) & 0xFF;
                        g_u16SPI2RxTail &= RX_BOUND_PART;

                        g_u8SPI2RxBuf[g_u16SPI2RxTail++] = (Data >> 16) & 0xFF;
                        g_u16SPI2RxTail &= RX_BOUND_PART;

                        g_u16SPI2RxByte += 3;
                        dataRxCnt--;
                    }
                }
                break;
            case 4:
                while(dataCnt || dataRxCnt) {

                    while(dataCnt && (0 == SPI_GET_TX_FULL(SPI2))) {

                        Data = g_u8SPI2TxBuf[g_u16SPI2TxHead++];
                        Data += (g_u8SPI2TxBuf[g_u16SPI2TxHead++] << 8);
                        Data += (g_u8SPI2TxBuf[g_u16SPI2TxHead++] << 16);
                        Data += (g_u8SPI2TxBuf[g_u16SPI2TxHead++] << 24);
                        g_u16SPI2TxByte -= 4;

                        SPI_WRITE_TX0(SPI2, Data);
                        dataCnt--;

                        if(0 == SPI_GET_RX_EMPTY(SPI2))
                            break;
                    }

                    while(dataRxCnt) {
                        while(SPI_GET_RX_EMPTY(SPI2));
                        Data = SPI_GET_RX0_DATA(SPI2);

                        g_u8SPI2RxBuf[g_u16SPI2RxTail++] = Data & 0xFF;
                        g_u16SPI2RxTail &= RX_BOUND_PART;

                        g_u8SPI2RxBuf[g_u16SPI2RxTail++] = (Data >> 8) & 0xFF;
                        g_u16SPI2RxTail &= RX_BOUND_PART;

                        g_u8SPI2RxBuf[g_u16SPI2RxTail++] = (Data >> 16) & 0xFF;
                        g_u16SPI2RxTail &= RX_BOUND_PART;

                        g_u8SPI2RxBuf[g_u16SPI2RxTail++] = (Data >> 24) & 0xFF;
                        g_u16SPI2RxTail &= RX_BOUND_PART;

                        g_u16SPI2RxByte += 4;
                        dataRxCnt--;
                    }
                }
            default:
                break;
            }

            if('P' == g_u8SPI2TxBuf[g_u16SPI2TxHead]) {
                SPI_CLR_SS0(SPI2);
                g_u16SPI2TxHead++;
            } else
                goto SPI_CHANGE_BIT_LENGTH;

            if(idleTimeL)
                SYS_SysTickDelay(idleTimeL);

            repeat--;
        }

        // Command Complete
        cmdCnt--;
    }
}




