/**************************************************************************//**
 * @file     SPI_Monitor.c
 * @version  V1.00
 * $Revision: 1 $
 * $Date: 13/11/11 11:27a $
 * @brief    NUC123 Series SPI Slave With Two Bit Mode and FIFO Enable Sample Code
 *
 * @note
 * Copyright (C) 2013 Nuvoton Technology Corp. All rights reserved.
 *
 ******************************************************************************/

/*!<Includes */
#include "NucInternal.h"

//#define           MODE_SPI_MONITOR        0x02

/**
  * @brief  Initial SPI0 to be SPI slave with default setting 0xF4.
  * @param  None.
  * @retval None.
  */
void SPI_Slave_Init(void)
{
    SPI0_Monitor_ReInit(0xF4);
}

/**
  * @brief  SPI0 main interrupt function.
  *             store received data in g_u8RxBuf, update related queue pointer and queue size
  * @param  None.
  * @retval None.
  */
void SPI0_IRQHandler(void)
{

    while(!SPI_GET_RX_EMPTY(SPI0)) {
        g_u8RxBuf[g_u16RxTail++] = SPI0->RX[0];
        g_u8RxBuf[g_u16RxTail++] = SPI0->RX[0];

        g_u16RxTail &= RX_BOUND;
        g_u16RxByte += 2;

        if(g_u16RxByte >  RX_BOUND) {
            g_u16RxBufferOverflow = 1;
            NVIC_DisableIRQ(SPI0_IRQn);
            NVIC_DisableIRQ(GPAB_IRQn);
        }

    }
    SPI_CLEAR_IF(SPI0);
}

/**
  * @brief  GPAB main interrupt function.
  *             store RxTail in buffer when SS in dis-active
  * @param  None.
  * @retval None.
  */
void GPAB_IRQHandler(void)
{
    GPIO_CLR_PIN_IF(PB, 3);

    g_u16CS_LabelBuf[g_u16CS_LabelTail++] = g_u16RxTail+SPI_GET_RX_FIFO_COUNT(SPI0);

    g_u16CS_LabelTail &= CS_LABEL_BOUND;
    g_u16CS_LabelByte += 1;

    if(g_u16CS_LabelByte >  CS_LABEL_BOUND) {
        g_u16CS_OverFlow = 1;
        NVIC_DisableIRQ(SPI0_IRQn);
        NVIC_DisableIRQ(GPAB_IRQn);
    }
}

/**
  * @brief  Disable SPI0 function and clock source. Set PC.0, PC.1, PC.3, and PC.5 as GPIO function
  * @param  None.
  * @retval None.
  */
void SPI_Slave_DeInit(void)
{
    NVIC_DisableIRQ(GPAB_IRQn);
    NVIC_DisableIRQ(SPI0_IRQn);
    DrvSPI_DeInit(SPI0);

    outpw(&SYS->GPC_MFP, inpw(&SYS->GPC_MFP) & ~(0x1<<0));
    outpw(&SYS->ALT_MFP, inpw(&SYS->ALT_MFP) & ~(0x1<<5));

    outpw(&SYS->GPC_MFP, inpw(&SYS->GPC_MFP) & ~(0x1<<1));
    outpw(&SYS->ALT_MFP, inpw(&SYS->ALT_MFP) & ~(0x1<<6));

    outpw(&SYS->GPC_MFP, inpw(&SYS->GPC_MFP) & ~(0x1<<3));
    outpw(&SYS->ALT_MFP, inpw(&SYS->ALT_MFP) & ~(0x1<<8));

    outpw(&SYS->GPC_MFP, inpw(&SYS->GPC_MFP) &~(0x1<<5));
    outpw(&SYS->ALT_MFP, inpw(&SYS->ALT_MFP) &~(0x1<<30));

    // Clear SPI0 Monitor recorded setting
    g_u8ParamSet[MODE_SPI_MONITOR] = 0x00;

}

/**
  * @brief  Initial SPI0 to be SPI slave.
  *              Set PC.0 as SPI0 SS.
  *              Set PC.1 as SPI0 CLK.
  *              Set PC.3 as SPI0 MOSI0.
  *              Set PC.5 as SPI0 MOSI1.    // No MISO pin, using SPI two bit mode
  * @param  setting: the setting of SPI
  *                 bit 0~1 (SPI_TRANS_TYPE)
  *                     -00: Type0
  *                     -01: Type1
  *                     -10: Type2
  *                     -11: Type3
  *                 bit 2
  *                     -0: Least Significant Bit first
  *                     -1: Most Significant Bit first
  *                 bit 3
  *                     -0: SS active Low
  *                     -1: SS active High
  *                 bit  4~7 (clock rate)
  *                     -1111: 12 MHz    // Always using 12 MHz
  *                     -others: Reserved
  * @retval None.
  */
void SPI0_Monitor_ReInit(uint8_t setting)
{

    GPIO_SPI0_SS0_PC0();
    GPIO_SPI0_CLK_PC1();
    GPIO_SPI0_MOSI0_PC3();
    outpw(&SYS->GPC_MFP, inpw(&SYS->GPC_MFP) | (0x1<<5));
    outpw(&SYS->ALT_MFP, inpw(&SYS->ALT_MFP) & ~(0x1<<30));

    // Bit 0, Bit 1 for SPI_TRANS_TYPE
    switch(setting & 0x03) {
    case 0:
        DrvSPI_Init(SPI0, SPI_SLAVE, SPI_TYPE0 , 8);
        break;

    case 1:
        DrvSPI_Init(SPI0, SPI_SLAVE, SPI_TYPE1 , 8);
        break;

    case 2:
        DrvSPI_Init(SPI0, SPI_SLAVE, SPI_TYPE2 , 8);
        break;

    case 3:
        DrvSPI_Init(SPI0, SPI_SLAVE, SPI_TYPE3 , 8);
        break;
    }



    SPI_SET_DIVIDER(SPI0, 6);
    SET_SPI0_CLKSRC(CLK_SPI0_PLL);

    DrvSPI_SetClockFreq(SPI0 , 12000000 , 0);

    SPI_ENABLE_TWO_BIT(SPI0);


    if(setting & 0x04)
        SPI_SET_MSB_FIRST(SPI0);
    else
        SPI_SET_LSB_FIRST(SPI0);

    SPI_SET_SS0(SPI0);

    if(setting & 0x08)
        SPI_SET_SS(SPI0, SPI_SS_HIGH_TRIGGER|SPI_SS_LEVEL_TRIGGER, SPI_SW_SS/*SPI_AUTO_SS*/); //En: Set slave auto select, high active
    else
        SPI_SET_SS(SPI0, SPI_SS_LOW_TRIGGER|SPI_SS_LEVEL_TRIGGER, SPI_SW_SS/*SPI_AUTO_SS*/); //En: Set slave auto select, low active


    SPI_SetFIFOMode(SPI0, 2);

    SPI_SET_RX_FIFO_THRESHOLD(SPI0 , 6);

    SPI_CLEAR_RX_FIFO(SPI0);

    NVIC_EnableIRQ(SPI0_IRQn);

    SPI_ENABLE_RX_THRESHOLD_INT(SPI0);

    SPI_ENABLE_TIMEOUT_INT(SPI0);

    SPI_SET_GO(SPI0);

    g_u32SPI0ApbClock = DrvSPI_GetClock1Freq(SPI0);

    g_u8ParamSet[MODE_SPI_MONITOR] = setting;

    GPIO_SET_MODE(PB, 3, GPIO_INPUT);
    if(setting & 0x08)
        GPIO_EnableInt(PB, 3, GPIO_INT_FALLING);
    else
        GPIO_EnableInt(PB, 3, GPIO_INT_RISING);

    NVIC_EnableIRQ(GPAB_IRQn);                          //En:Enable NVIC GPAB_IRQn interrupt

}
